


#ifdef NMI_TEST
# include "1.0.2/rpctest.h"

/*work aroundS for header file conflicts*/

# define bool_t bool_t_rpc
# undef T_ERROR
#endif

#include <rpc/rpcb_clnt.h>


#ifdef JNI_TEST
# include "rpctest.h"
#include "jni.h"
#endif


enum RpcTestErrors
{ RPCTEST_NO_ERROR=0,
  RPCTEST_NO_NETCONFIG=1,
  RPCTEST_NO_GETMAPS=2,
  RPCTEST_NO_PROGNUM=3,
  RPCTEST_NO_MALLOC=4
};

static enum RpcTestErrors MatchProgNum(char** outString,
				       unsigned long progNum);

static const char* RpcErrToString(enum RpcTestErrors curErr);

#ifdef JNI_TEST

JNIEXPORT jstring
JNICALL Java_rpctest_GetProgNumInfo(JNIEnv * curEnv,
				    jclass curClass,
				    jlong progNum)
{
  jstring outString=(jstring)NULL;
  char *resString=(char*)NULL;
  const char* fatalString=NULL;
  
  enum RpcTestErrors  result=MatchProgNum(&resString,
					  progNum);

  if (result==RPCTEST_NO_ERROR)
    {
      outString=(*curEnv)->NewStringUTF(curEnv,resString);
      free(resString);
    }
  else
    {
      fatalString=RpcErrToString(result);
    }
  
  if (fatalString)
    (*curEnv)->FatalError(curEnv,fatalString);
  
  return outString;
}


#endif

static enum RpcTestErrors MatchProgNum(char** outString,
				       unsigned long progNum)

{
  short fFound=FALSE;
  struct netbuf svcaddr;
  
  struct netconfig* curNetConfig=getnetconfigent(NC_TCP);


  rpcblist *curList=(rpcblist*)NULL;
  rpcblist *origList=(rpcblist*)NULL;

  if (!curNetConfig)
    {
      return RPCTEST_NO_NETCONFIG;
    }

  curList=rpcb_getmaps(curNetConfig, "localhost");
  origList=curList;
  
  if (!curList)
    {
      freenetconfigent(curNetConfig);
      return RPCTEST_NO_GETMAPS;
    }
  
  fFound=FALSE;
  while (curList && (!fFound))
    {
      if ((strcmp(curList->rpcb_map.r_netid,NC_TCP)==0) &&
	  (curList->rpcb_map.r_prog==progNum))
	{
	  fFound=TRUE;
	}
      else
	{
	  curList=curList->rpcb_next;
	}
    }

  if (fFound)
  {
    *outString=(char*)malloc(strlen(curList->rpcb_map.r_addr)+1);
    if (!*outString)
      {
	free(origList);
  	freenetconfigent(curNetConfig);
	return RPCTEST_NO_MALLOC;
      }
    strcpy(*outString,curList->rpcb_map.r_addr);
  }
  
  if (origList)
    free(origList);
  
  if (curNetConfig)
    freenetconfigent(curNetConfig);

  if (!fFound)
    {
      return RPCTEST_NO_PROGNUM;
    }
  
  return RPCTEST_NO_ERROR;
}


#ifdef CTEST
int
main(int argc, char** argv)
{
  unsigned int value;
  char *outString;
  enum RpcTestErrors result;
  
  if (argc!=2)
    {
      fputs("Usage: rpcctest prognum\n",stderr);
      exit(1);
    }

  value=atol(argv[1]);

  result=MatchProgNum(&outString,value);
  if (result != RPCTEST_NO_ERROR)
    {
      fprintf(stderr,"rpctest internal error #%d:\n\t%s\n",
	      (int)result,
	      RpcErrToString(result));
      exit(1);
    }
  printf("Found prognum with address:%s\n", outString);
  free(outString);
  
  return 0;
}

#endif


static const char*
RpcErrToString(enum RpcTestErrors curErr)
{
  switch (curErr)
    {
    case RPCTEST_NO_ERROR:
      return "No Error";
      break;
      
    case RPCTEST_NO_NETCONFIG:
      return "Could not get the netconfig";
      break;
      
    case RPCTEST_NO_GETMAPS:
      return "rpcb_getmaps failed";
      break;
      
    case RPCTEST_NO_PROGNUM:
      return "Prognum not found";
      break;

    case RPCTEST_NO_MALLOC:
      return "malloc failed";
      break;
      
    default:
      return "Unknown RPCTEST error";
    }
    
    return "Internal RPCTEST error";
}

#ifdef NMI_TEST 
/* Old 1.0.2 Java calling conventions */


extern struct Hjava_lang_String*
rpctest_GetProgNumInfo(struct Hrpctest *curClass,
		       int64_t progNum)
{
  Hjava_lang_String* outString=(Hjava_lang_String*)NULL;
  char *resString=(char*)NULL;
  const char* fatalString=NULL;
  
  enum RpcTestErrors  result=MatchProgNum(&resString,
					  progNum);

  if (result==RPCTEST_NO_ERROR)
    {
      outString=makeJavaString((char*)resString,strlen(resString));
      free(resString);
    }
  else
    {
      fatalString=RpcErrToString(result);
    }
  
  if (fatalString)
    SignalError(0,"java/lang/RuntimeException",
		(char*)fatalString);
  
  return outString;
}


#endif
